from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event import Event


T = TypeVar("T", bound="NotificationUpdate")


@_attrs_define
class NotificationUpdate:
    """
    Attributes:
        duid (str):
        user_duid (Union[Unset, str]):
        event (Union[Unset, Event]):
        read (Union[Unset, bool]):
    """

    duid: str
    user_duid: Union[Unset, str] = UNSET
    event: Union[Unset, "Event"] = UNSET
    read: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        user_duid = self.user_duid

        event: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.event, Unset):
            event = self.event.to_dict()

        read = self.read

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if user_duid is not UNSET:
            field_dict["userDuid"] = user_duid
        if event is not UNSET:
            field_dict["event"] = event
        if read is not UNSET:
            field_dict["read"] = read

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event import Event

        d = src_dict.copy()
        duid = d.pop("duid")

        user_duid = d.pop("userDuid", UNSET)

        _event = d.pop("event", UNSET)
        event: Union[Unset, Event]
        if isinstance(_event, Unset):
            event = UNSET
        else:
            event = Event.from_dict(_event)

        read = d.pop("read", UNSET)

        notification_update = cls(
            duid=duid,
            user_duid=user_duid,
            event=event,
            read=read,
        )

        notification_update.additional_properties = d
        return notification_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from .generated.models import *
from .dart import (
    Dart,
    cli,
    set_host,
    is_logged_in,
    login,
    create_task,
    update_task,
    begin_task,
    replicate_space,
)
from .webhook import is_signature_correct



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.color_name import ColorName
from ..models.theme import Theme
from ..models.user_role import UserRole
from ..models.user_status import UserStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_data import GoogleData


T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """
    Attributes:
        duid (str):
        status (UserStatus): * `PendingSubscriptionUpgrade` - PENDING_SUBSCRIPTION_UPGRADE
            * `Invited` - INVITED
            * `PendingEmailVerification` - PENDING_EMAIL_VERIFICATION
            * `Active` - ACTIVE
            * `Deactivated` - DEACTIVATED
        role (UserRole): * `Admin` - ADMIN
            * `Member` - MEMBER
            * `Guest` - GUEST
        email (str):
        name (str):
        abrev (str):
        theme (Theme): * `System Default` - SYSTEM_DEFAULT
            * `Light` - LIGHT
            * `Dark` - DARK
        color_hex (str):
        color_name (ColorName): * `Red` - RED
            * `Dark Blue` - DARK_BLUE
            * `Dark Orange` - DARK_ORANGE
            * `Dark Green` - DARK_GREEN
            * `Purple` - PURPLE
            * `Dark Teal` - DARK_TEAL
            * `Pink` - PINK
            * `Orange` - ORANGE
            * `Green` - GREEN
            * `Yellow` - YELLOW
            * `Brown` - BROWN
            * `Dark Red` - DARK_RED
            * `Flat Green` - FLAT_GREEN
            * `Red Orange` - RED_ORANGE
            * `Teal` - TEAL
            * `Light Green` - LIGHT_GREEN
            * `Light Blue` - LIGHT_BLUE
            * `Light Purple` - LIGHT_PURPLE
            * `Light Orange` - LIGHT_ORANGE
            * `Light Pink` - LIGHT_PINK
            * `Tan` - TAN
            * `Dark Gray` - DARK_GRAY
            * `Light Brown` - LIGHT_BROWN
            * `Light Gray` - LIGHT_GRAY
        sections (Any):
        layout (Any):
        image_url (Union[None, str]):
        is_admin (bool):
        auth_token (Union[None, str]):
        google_data (Union['GoogleData', None]):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    status: UserStatus
    role: UserRole
    email: str
    name: str
    abrev: str
    theme: Theme
    color_hex: str
    color_name: ColorName
    sections: Any
    layout: Any
    image_url: Union[None, str]
    is_admin: bool
    auth_token: Union[None, str]
    google_data: Union["GoogleData", None]
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.google_data import GoogleData

        duid = self.duid

        status = self.status.value

        role = self.role.value

        email = self.email

        name = self.name

        abrev = self.abrev

        theme = self.theme.value

        color_hex = self.color_hex

        color_name = self.color_name.value

        sections = self.sections

        layout = self.layout

        image_url: Union[None, str]
        image_url = self.image_url

        is_admin = self.is_admin

        auth_token: Union[None, str]
        auth_token = self.auth_token

        google_data: Union[Dict[str, Any], None]
        if isinstance(self.google_data, GoogleData):
            google_data = self.google_data.to_dict()
        else:
            google_data = self.google_data

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "status": status,
                "role": role,
                "email": email,
                "name": name,
                "abrev": abrev,
                "theme": theme,
                "colorHex": color_hex,
                "colorName": color_name,
                "sections": sections,
                "layout": layout,
                "imageUrl": image_url,
                "isAdmin": is_admin,
                "authToken": auth_token,
                "googleData": google_data,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_data import GoogleData

        d = src_dict.copy()
        duid = d.pop("duid")

        status = UserStatus(d.pop("status"))

        role = UserRole(d.pop("role"))

        email = d.pop("email")

        name = d.pop("name")

        abrev = d.pop("abrev")

        theme = Theme(d.pop("theme"))

        color_hex = d.pop("colorHex")

        color_name = ColorName(d.pop("colorName"))

        sections = d.pop("sections")

        layout = d.pop("layout")

        def _parse_image_url(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        image_url = _parse_image_url(d.pop("imageUrl"))

        is_admin = d.pop("isAdmin")

        def _parse_auth_token(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        auth_token = _parse_auth_token(d.pop("authToken"))

        def _parse_google_data(data: object) -> Union["GoogleData", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                google_data_type_0 = GoogleData.from_dict(data)

                return google_data_type_0
            except:  # noqa: E722
                pass
            return cast(Union["GoogleData", None], data)

        google_data = _parse_google_data(d.pop("googleData"))

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        user = cls(
            duid=duid,
            status=status,
            role=role,
            email=email,
            name=name,
            abrev=abrev,
            theme=theme,
            color_hex=color_hex,
            color_name=color_name,
            sections=sections,
            layout=layout,
            image_url=image_url,
            is_admin=is_admin,
            auth_token=auth_token,
            google_data=google_data,
            updated_by_client_duid=updated_by_client_duid,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.priority import Priority
from ..models.task_source_type import TaskSourceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.relationship import Relationship
    from ..models.task_link import TaskLink
    from ..models.task_notion_document import TaskNotionDocument
    from ..models.task_properties import TaskProperties


T = TypeVar("T", bound="Task")


@_attrs_define
class Task:
    """
    Attributes:
        duid (str):
        source_type (TaskSourceType): * `Unknown` - UNKNOWN
            * `Import` - IMPORT
            * `Onboarding` - ONBOARDING
            * `Recommendation` - RECOMMENDATION
            * `Recurrence` - RECURRENCE
            * `Template` - TEMPLATE
            * `ChatGPT` - CHAT_GPT
            * `Email` - EMAIL
            * `Slack` - SLACK
            * `API` - API
            * `CLI` - CLI
            * `Application` - APPLICATION
            * `AppTcm` - APP_TCM
            * `AppInternalForm` - APP_INTERNAL_FORM
            * `AppQuickAdd` - APP_QUICK_ADD
            * `AppBoard` - APP_BOARD
            * `AppSubtask` - APP_SUBTASK
            * `AppRelationship` - APP_RELATIONSHIP
            * `AppEnter` - APP_ENTER
            * `AppReplicate` - APP_REPLICATE
            * `AppPaste` - APP_PASTE
            * `AppRoadmapList` - APP_ROADMAP_LIST
            * `AppRoadmapTimeline` - APP_ROADMAP_TIMELINE
            * `ExternalForm` - EXTERNAL_FORM
        created_at (datetime.datetime):
        created_by_duid (Union[None, str]):
        updated_at (datetime.datetime):
        updated_by_duid (Union[None, str]):
        drafter_duid (Union[None, str]):
        in_trash (bool):
        dartboard_duid (str):
        order (str):
        expanded (bool):
        title (str):
        description (Any):
        notion_document (Union['TaskNotionDocument', None]):
        status_duid (str):
        assigned_to_ai (bool):
        recommendation_duid (Union[None, str]):
        assignee_duids (List[str]):
        subscriber_duids (List[str]):
        tag_duids (List[str]):
        links (List['TaskLink']):
        attachment_duids (List[str]):
        relationships (List['Relationship']):
        priority (Union[None, Priority]):
        size (Union[None, int]):
        start_at (Union[None, datetime.datetime]):
        due_at (Union[None, datetime.datetime]):
        remind_at (Union[None, datetime.datetime]):
        recurrence (Union[Any, None]):
        recurs_next_at (Union[None, datetime.datetime]):
        properties (TaskProperties):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    source_type: TaskSourceType
    created_at: datetime.datetime
    created_by_duid: Union[None, str]
    updated_at: datetime.datetime
    updated_by_duid: Union[None, str]
    drafter_duid: Union[None, str]
    in_trash: bool
    dartboard_duid: str
    order: str
    expanded: bool
    title: str
    description: Any
    notion_document: Union["TaskNotionDocument", None]
    status_duid: str
    assigned_to_ai: bool
    recommendation_duid: Union[None, str]
    assignee_duids: List[str]
    subscriber_duids: List[str]
    tag_duids: List[str]
    links: List["TaskLink"]
    attachment_duids: List[str]
    relationships: List["Relationship"]
    priority: Union[None, Priority]
    size: Union[None, int]
    start_at: Union[None, datetime.datetime]
    due_at: Union[None, datetime.datetime]
    remind_at: Union[None, datetime.datetime]
    recurrence: Union[Any, None]
    recurs_next_at: Union[None, datetime.datetime]
    properties: "TaskProperties"
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.task_notion_document import TaskNotionDocument

        duid = self.duid

        source_type = self.source_type.value

        created_at = self.created_at.isoformat()

        created_by_duid: Union[None, str]
        created_by_duid = self.created_by_duid

        updated_at = self.updated_at.isoformat()

        updated_by_duid: Union[None, str]
        updated_by_duid = self.updated_by_duid

        drafter_duid: Union[None, str]
        drafter_duid = self.drafter_duid

        in_trash = self.in_trash

        dartboard_duid = self.dartboard_duid

        order = self.order

        expanded = self.expanded

        title = self.title

        description = self.description

        notion_document: Union[Dict[str, Any], None]
        if isinstance(self.notion_document, TaskNotionDocument):
            notion_document = self.notion_document.to_dict()
        else:
            notion_document = self.notion_document

        status_duid = self.status_duid

        assigned_to_ai = self.assigned_to_ai

        recommendation_duid: Union[None, str]
        recommendation_duid = self.recommendation_duid

        assignee_duids = self.assignee_duids

        subscriber_duids = self.subscriber_duids

        tag_duids = self.tag_duids

        links = []
        for links_item_data in self.links:
            links_item = links_item_data.to_dict()
            links.append(links_item)

        attachment_duids = self.attachment_duids

        relationships = []
        for relationships_item_data in self.relationships:
            relationships_item = relationships_item_data.to_dict()
            relationships.append(relationships_item)

        priority: Union[None, str]
        if isinstance(self.priority, Priority):
            priority = self.priority.value
        else:
            priority = self.priority

        size: Union[None, int]
        size = self.size

        start_at: Union[None, str]
        if isinstance(self.start_at, datetime.datetime):
            start_at = self.start_at.isoformat()
        else:
            start_at = self.start_at

        due_at: Union[None, str]
        if isinstance(self.due_at, datetime.datetime):
            due_at = self.due_at.isoformat()
        else:
            due_at = self.due_at

        remind_at: Union[None, str]
        if isinstance(self.remind_at, datetime.datetime):
            remind_at = self.remind_at.isoformat()
        else:
            remind_at = self.remind_at

        recurrence: Union[Any, None]
        recurrence = self.recurrence

        recurs_next_at: Union[None, str]
        if isinstance(self.recurs_next_at, datetime.datetime):
            recurs_next_at = self.recurs_next_at.isoformat()
        else:
            recurs_next_at = self.recurs_next_at

        properties = self.properties.to_dict()

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "sourceType": source_type,
                "createdAt": created_at,
                "createdByDuid": created_by_duid,
                "updatedAt": updated_at,
                "updatedByDuid": updated_by_duid,
                "drafterDuid": drafter_duid,
                "inTrash": in_trash,
                "dartboardDuid": dartboard_duid,
                "order": order,
                "expanded": expanded,
                "title": title,
                "description": description,
                "notionDocument": notion_document,
                "statusDuid": status_duid,
                "assignedToAi": assigned_to_ai,
                "recommendationDuid": recommendation_duid,
                "assigneeDuids": assignee_duids,
                "subscriberDuids": subscriber_duids,
                "tagDuids": tag_duids,
                "links": links,
                "attachmentDuids": attachment_duids,
                "relationships": relationships,
                "priority": priority,
                "size": size,
                "startAt": start_at,
                "dueAt": due_at,
                "remindAt": remind_at,
                "recurrence": recurrence,
                "recursNextAt": recurs_next_at,
                "properties": properties,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.relationship import Relationship
        from ..models.task_link import TaskLink
        from ..models.task_notion_document import TaskNotionDocument
        from ..models.task_properties import TaskProperties

        d = src_dict.copy()
        duid = d.pop("duid")

        source_type = TaskSourceType(d.pop("sourceType"))

        created_at = isoparse(d.pop("createdAt"))

        def _parse_created_by_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        created_by_duid = _parse_created_by_duid(d.pop("createdByDuid"))

        updated_at = isoparse(d.pop("updatedAt"))

        def _parse_updated_by_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        updated_by_duid = _parse_updated_by_duid(d.pop("updatedByDuid"))

        def _parse_drafter_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        drafter_duid = _parse_drafter_duid(d.pop("drafterDuid"))

        in_trash = d.pop("inTrash")

        dartboard_duid = d.pop("dartboardDuid")

        order = d.pop("order")

        expanded = d.pop("expanded")

        title = d.pop("title")

        description = d.pop("description")

        def _parse_notion_document(data: object) -> Union["TaskNotionDocument", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                notion_document_type_0 = TaskNotionDocument.from_dict(data)

                return notion_document_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TaskNotionDocument", None], data)

        notion_document = _parse_notion_document(d.pop("notionDocument"))

        status_duid = d.pop("statusDuid")

        assigned_to_ai = d.pop("assignedToAi")

        def _parse_recommendation_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        recommendation_duid = _parse_recommendation_duid(d.pop("recommendationDuid"))

        assignee_duids = cast(List[str], d.pop("assigneeDuids"))

        subscriber_duids = cast(List[str], d.pop("subscriberDuids"))

        tag_duids = cast(List[str], d.pop("tagDuids"))

        links = []
        _links = d.pop("links")
        for links_item_data in _links:
            links_item = TaskLink.from_dict(links_item_data)

            links.append(links_item)

        attachment_duids = cast(List[str], d.pop("attachmentDuids"))

        relationships = []
        _relationships = d.pop("relationships")
        for relationships_item_data in _relationships:
            relationships_item = Relationship.from_dict(relationships_item_data)

            relationships.append(relationships_item)

        def _parse_priority(data: object) -> Union[None, Priority]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                priority_type_0 = Priority(data)

                return priority_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Priority], data)

        priority = _parse_priority(d.pop("priority"))

        def _parse_size(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        size = _parse_size(d.pop("size"))

        def _parse_start_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_at_type_0 = isoparse(data)

                return start_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        start_at = _parse_start_at(d.pop("startAt"))

        def _parse_due_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                due_at_type_0 = isoparse(data)

                return due_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        due_at = _parse_due_at(d.pop("dueAt"))

        def _parse_remind_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                remind_at_type_0 = isoparse(data)

                return remind_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        remind_at = _parse_remind_at(d.pop("remindAt"))

        def _parse_recurrence(data: object) -> Union[Any, None]:
            if data is None:
                return data
            return cast(Union[Any, None], data)

        recurrence = _parse_recurrence(d.pop("recurrence"))

        def _parse_recurs_next_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                recurs_next_at_type_0 = isoparse(data)

                return recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        recurs_next_at = _parse_recurs_next_at(d.pop("recursNextAt"))

        properties = TaskProperties.from_dict(d.pop("properties"))

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        task = cls(
            duid=duid,
            source_type=source_type,
            created_at=created_at,
            created_by_duid=created_by_duid,
            updated_at=updated_at,
            updated_by_duid=updated_by_duid,
            drafter_duid=drafter_duid,
            in_trash=in_trash,
            dartboard_duid=dartboard_duid,
            order=order,
            expanded=expanded,
            title=title,
            description=description,
            notion_document=notion_document,
            status_duid=status_duid,
            assigned_to_ai=assigned_to_ai,
            recommendation_duid=recommendation_duid,
            assignee_duids=assignee_duids,
            subscriber_duids=subscriber_duids,
            tag_duids=tag_duids,
            links=links,
            attachment_duids=attachment_duids,
            relationships=relationships,
            priority=priority,
            size=size,
            start_at=start_at,
            due_at=due_at,
            remind_at=remind_at,
            recurrence=recurrence,
            recurs_next_at=recurs_next_at,
            properties=properties,
            updated_by_client_duid=updated_by_client_duid,
        )

        task.additional_properties = d
        return task

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.brainstorm_state import BrainstormState
from ..types import UNSET, Unset

T = TypeVar("T", bound="BrainstormUpdate")


@_attrs_define
class BrainstormUpdate:
    """
    Attributes:
        duid (str):
        dartboard_duid (Union[Unset, str]):
        subject (Union[Unset, str]):
        ai (Union[Unset, bool]):
        total_session_ms (Union[Unset, int]):
        started_at (Union[Unset, datetime.datetime]):
        state (Union[Unset, BrainstormState]): * `Running` - RUNNING
            * `Paused` - PAUSED
            * `Stopped` - STOPPED
        after_start_ms (Union[Unset, int]):
        created_tasks_duids (Union[Unset, List[str]]):
    """

    duid: str
    dartboard_duid: Union[Unset, str] = UNSET
    subject: Union[Unset, str] = UNSET
    ai: Union[Unset, bool] = UNSET
    total_session_ms: Union[Unset, int] = UNSET
    started_at: Union[Unset, datetime.datetime] = UNSET
    state: Union[Unset, BrainstormState] = UNSET
    after_start_ms: Union[Unset, int] = UNSET
    created_tasks_duids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        dartboard_duid = self.dartboard_duid

        subject = self.subject

        ai = self.ai

        total_session_ms = self.total_session_ms

        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        after_start_ms = self.after_start_ms

        created_tasks_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.created_tasks_duids, Unset):
            created_tasks_duids = self.created_tasks_duids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if dartboard_duid is not UNSET:
            field_dict["dartboardDuid"] = dartboard_duid
        if subject is not UNSET:
            field_dict["subject"] = subject
        if ai is not UNSET:
            field_dict["ai"] = ai
        if total_session_ms is not UNSET:
            field_dict["totalSessionMs"] = total_session_ms
        if started_at is not UNSET:
            field_dict["startedAt"] = started_at
        if state is not UNSET:
            field_dict["state"] = state
        if after_start_ms is not UNSET:
            field_dict["afterStartMs"] = after_start_ms
        if created_tasks_duids is not UNSET:
            field_dict["createdTasksDuids"] = created_tasks_duids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        dartboard_duid = d.pop("dartboardDuid", UNSET)

        subject = d.pop("subject", UNSET)

        ai = d.pop("ai", UNSET)

        total_session_ms = d.pop("totalSessionMs", UNSET)

        _started_at = d.pop("startedAt", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _state = d.pop("state", UNSET)
        state: Union[Unset, BrainstormState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = BrainstormState(_state)

        after_start_ms = d.pop("afterStartMs", UNSET)

        created_tasks_duids = cast(List[str], d.pop("createdTasksDuids", UNSET))

        brainstorm_update = cls(
            duid=duid,
            dartboard_duid=dartboard_duid,
            subject=subject,
            ai=ai,
            total_session_ms=total_session_ms,
            started_at=started_at,
            state=state,
            after_start_ms=after_start_ms,
            created_tasks_duids=created_tasks_duids,
        )

        brainstorm_update.additional_properties = d
        return brainstorm_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import hashlib
import hmac
import os


_WEBHOOK_SECRET = os.environ.get("DART_WEBHOOK_SECRET")
if _WEBHOOK_SECRET is not None:
    _WEBHOOK_SECRET = _WEBHOOK_SECRET.encode("utf-8")


def is_signature_correct(payload: bytes, signature: str) -> bool:
    if _WEBHOOK_SECRET is None:
        raise RuntimeError("DART_WEBHOOK_SECRET environment variable is not set")
    expected_signature = hmac.new(_WEBHOOK_SECRET, payload, hashlib.sha256).hexdigest()
    try:
        return hmac.compare_digest(expected_signature, signature)
    except TypeError:
        return False



from enum import Enum


class EntityName(str, Enum):
    BRAINSTORM = "brainstorm"
    COMMENT = "comment"
    DARTBOARD = "dartboard"
    DOC = "doc"
    FOLDER = "folder"
    FORM = "form"
    PROPERTY = "property"
    SPACE = "space"
    STATUS = "status"
    TASK = "task"
    TENANT = "tenant"
    UNKNOWN = "UNKNOWN"
    USER = "user"
    VIEW = "view"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.priority import Priority
from ..models.task_source_type import TaskSourceType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TaskUpdate")


@_attrs_define
class TaskUpdate:
    """
    Attributes:
        duid (str):
        source_type (Union[Unset, TaskSourceType]): * `Unknown` - UNKNOWN
            * `Import` - IMPORT
            * `Onboarding` - ONBOARDING
            * `Recommendation` - RECOMMENDATION
            * `Recurrence` - RECURRENCE
            * `Template` - TEMPLATE
            * `ChatGPT` - CHAT_GPT
            * `Email` - EMAIL
            * `Slack` - SLACK
            * `API` - API
            * `CLI` - CLI
            * `Application` - APPLICATION
            * `AppTcm` - APP_TCM
            * `AppInternalForm` - APP_INTERNAL_FORM
            * `AppQuickAdd` - APP_QUICK_ADD
            * `AppBoard` - APP_BOARD
            * `AppSubtask` - APP_SUBTASK
            * `AppRelationship` - APP_RELATIONSHIP
            * `AppEnter` - APP_ENTER
            * `AppReplicate` - APP_REPLICATE
            * `AppPaste` - APP_PASTE
            * `AppRoadmapList` - APP_ROADMAP_LIST
            * `AppRoadmapTimeline` - APP_ROADMAP_TIMELINE
            * `ExternalForm` - EXTERNAL_FORM Default: TaskSourceType.UNKNOWN.
        source_template_view_duid (Union[None, Unset, str]):
        source_template_task_duid (Union[None, Unset, str]):
        source_form_duid (Union[None, Unset, str]):
        created_by_duid (Union[None, Unset, str]):
        updated_by_duid (Union[None, Unset, str]):
        drafter_duid (Union[None, Unset, str]):
        in_trash (Union[Unset, bool]):
        dartboard_duid (Union[Unset, str]):
        order (Union[Unset, str]):
        expanded (Union[Unset, bool]):
        title (Union[Unset, str]):
        description (Union[Unset, Any]):
        description_markdown (Union[Unset, str]):
        status_duid (Union[Unset, str]):
        assigned_to_ai (Union[Unset, bool]):
        recommendation_duid (Union[None, Unset, str]):
        assignee_duids (Union[Unset, List[str]]):
        subscriber_duids (Union[Unset, List[str]]):
        tag_duids (Union[Unset, List[str]]):
        attachment_duids (Union[Unset, List[str]]):
        priority (Union[None, Priority, Unset]):
        size (Union[None, Unset, int]):
        start_at (Union[None, Unset, datetime.datetime]):
        due_at (Union[None, Unset, datetime.datetime]):
        remind_at (Union[None, Unset, datetime.datetime]):
        recurrence (Union[Any, None, Unset]):
        recurs_next_at (Union[None, Unset, datetime.datetime]):
        properties (Union[Unset, Any]):
    """

    duid: str
    source_type: Union[Unset, TaskSourceType] = TaskSourceType.UNKNOWN
    source_template_view_duid: Union[None, Unset, str] = UNSET
    source_template_task_duid: Union[None, Unset, str] = UNSET
    source_form_duid: Union[None, Unset, str] = UNSET
    created_by_duid: Union[None, Unset, str] = UNSET
    updated_by_duid: Union[None, Unset, str] = UNSET
    drafter_duid: Union[None, Unset, str] = UNSET
    in_trash: Union[Unset, bool] = UNSET
    dartboard_duid: Union[Unset, str] = UNSET
    order: Union[Unset, str] = UNSET
    expanded: Union[Unset, bool] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, Any] = UNSET
    description_markdown: Union[Unset, str] = UNSET
    status_duid: Union[Unset, str] = UNSET
    assigned_to_ai: Union[Unset, bool] = UNSET
    recommendation_duid: Union[None, Unset, str] = UNSET
    assignee_duids: Union[Unset, List[str]] = UNSET
    subscriber_duids: Union[Unset, List[str]] = UNSET
    tag_duids: Union[Unset, List[str]] = UNSET
    attachment_duids: Union[Unset, List[str]] = UNSET
    priority: Union[None, Priority, Unset] = UNSET
    size: Union[None, Unset, int] = UNSET
    start_at: Union[None, Unset, datetime.datetime] = UNSET
    due_at: Union[None, Unset, datetime.datetime] = UNSET
    remind_at: Union[None, Unset, datetime.datetime] = UNSET
    recurrence: Union[Any, None, Unset] = UNSET
    recurs_next_at: Union[None, Unset, datetime.datetime] = UNSET
    properties: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        source_type: Union[Unset, str] = UNSET
        if not isinstance(self.source_type, Unset):
            source_type = self.source_type.value

        source_template_view_duid: Union[None, Unset, str]
        if isinstance(self.source_template_view_duid, Unset):
            source_template_view_duid = UNSET
        else:
            source_template_view_duid = self.source_template_view_duid

        source_template_task_duid: Union[None, Unset, str]
        if isinstance(self.source_template_task_duid, Unset):
            source_template_task_duid = UNSET
        else:
            source_template_task_duid = self.source_template_task_duid

        source_form_duid: Union[None, Unset, str]
        if isinstance(self.source_form_duid, Unset):
            source_form_duid = UNSET
        else:
            source_form_duid = self.source_form_duid

        created_by_duid: Union[None, Unset, str]
        if isinstance(self.created_by_duid, Unset):
            created_by_duid = UNSET
        else:
            created_by_duid = self.created_by_duid

        updated_by_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_duid, Unset):
            updated_by_duid = UNSET
        else:
            updated_by_duid = self.updated_by_duid

        drafter_duid: Union[None, Unset, str]
        if isinstance(self.drafter_duid, Unset):
            drafter_duid = UNSET
        else:
            drafter_duid = self.drafter_duid

        in_trash = self.in_trash

        dartboard_duid = self.dartboard_duid

        order = self.order

        expanded = self.expanded

        title = self.title

        description = self.description

        description_markdown = self.description_markdown

        status_duid = self.status_duid

        assigned_to_ai = self.assigned_to_ai

        recommendation_duid: Union[None, Unset, str]
        if isinstance(self.recommendation_duid, Unset):
            recommendation_duid = UNSET
        else:
            recommendation_duid = self.recommendation_duid

        assignee_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.assignee_duids, Unset):
            assignee_duids = self.assignee_duids

        subscriber_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.subscriber_duids, Unset):
            subscriber_duids = self.subscriber_duids

        tag_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tag_duids, Unset):
            tag_duids = self.tag_duids

        attachment_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.attachment_duids, Unset):
            attachment_duids = self.attachment_duids

        priority: Union[None, Unset, str]
        if isinstance(self.priority, Unset):
            priority = UNSET
        elif isinstance(self.priority, Priority):
            priority = self.priority.value
        else:
            priority = self.priority

        size: Union[None, Unset, int]
        if isinstance(self.size, Unset):
            size = UNSET
        else:
            size = self.size

        start_at: Union[None, Unset, str]
        if isinstance(self.start_at, Unset):
            start_at = UNSET
        elif isinstance(self.start_at, datetime.datetime):
            start_at = self.start_at.isoformat()
        else:
            start_at = self.start_at

        due_at: Union[None, Unset, str]
        if isinstance(self.due_at, Unset):
            due_at = UNSET
        elif isinstance(self.due_at, datetime.datetime):
            due_at = self.due_at.isoformat()
        else:
            due_at = self.due_at

        remind_at: Union[None, Unset, str]
        if isinstance(self.remind_at, Unset):
            remind_at = UNSET
        elif isinstance(self.remind_at, datetime.datetime):
            remind_at = self.remind_at.isoformat()
        else:
            remind_at = self.remind_at

        recurrence: Union[Any, None, Unset]
        if isinstance(self.recurrence, Unset):
            recurrence = UNSET
        else:
            recurrence = self.recurrence

        recurs_next_at: Union[None, Unset, str]
        if isinstance(self.recurs_next_at, Unset):
            recurs_next_at = UNSET
        elif isinstance(self.recurs_next_at, datetime.datetime):
            recurs_next_at = self.recurs_next_at.isoformat()
        else:
            recurs_next_at = self.recurs_next_at

        properties = self.properties

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if source_type is not UNSET:
            field_dict["sourceType"] = source_type
        if source_template_view_duid is not UNSET:
            field_dict["sourceTemplateViewDuid"] = source_template_view_duid
        if source_template_task_duid is not UNSET:
            field_dict["sourceTemplateTaskDuid"] = source_template_task_duid
        if source_form_duid is not UNSET:
            field_dict["sourceFormDuid"] = source_form_duid
        if created_by_duid is not UNSET:
            field_dict["createdByDuid"] = created_by_duid
        if updated_by_duid is not UNSET:
            field_dict["updatedByDuid"] = updated_by_duid
        if drafter_duid is not UNSET:
            field_dict["drafterDuid"] = drafter_duid
        if in_trash is not UNSET:
            field_dict["inTrash"] = in_trash
        if dartboard_duid is not UNSET:
            field_dict["dartboardDuid"] = dartboard_duid
        if order is not UNSET:
            field_dict["order"] = order
        if expanded is not UNSET:
            field_dict["expanded"] = expanded
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if description_markdown is not UNSET:
            field_dict["descriptionMarkdown"] = description_markdown
        if status_duid is not UNSET:
            field_dict["statusDuid"] = status_duid
        if assigned_to_ai is not UNSET:
            field_dict["assignedToAi"] = assigned_to_ai
        if recommendation_duid is not UNSET:
            field_dict["recommendationDuid"] = recommendation_duid
        if assignee_duids is not UNSET:
            field_dict["assigneeDuids"] = assignee_duids
        if subscriber_duids is not UNSET:
            field_dict["subscriberDuids"] = subscriber_duids
        if tag_duids is not UNSET:
            field_dict["tagDuids"] = tag_duids
        if attachment_duids is not UNSET:
            field_dict["attachmentDuids"] = attachment_duids
        if priority is not UNSET:
            field_dict["priority"] = priority
        if size is not UNSET:
            field_dict["size"] = size
        if start_at is not UNSET:
            field_dict["startAt"] = start_at
        if due_at is not UNSET:
            field_dict["dueAt"] = due_at
        if remind_at is not UNSET:
            field_dict["remindAt"] = remind_at
        if recurrence is not UNSET:
            field_dict["recurrence"] = recurrence
        if recurs_next_at is not UNSET:
            field_dict["recursNextAt"] = recurs_next_at
        if properties is not UNSET:
            field_dict["properties"] = properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        _source_type = d.pop("sourceType", UNSET)
        source_type: Union[Unset, TaskSourceType]
        if isinstance(_source_type, Unset):
            source_type = UNSET
        else:
            source_type = TaskSourceType(_source_type)

        def _parse_source_template_view_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        source_template_view_duid = _parse_source_template_view_duid(d.pop("sourceTemplateViewDuid", UNSET))

        def _parse_source_template_task_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        source_template_task_duid = _parse_source_template_task_duid(d.pop("sourceTemplateTaskDuid", UNSET))

        def _parse_source_form_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        source_form_duid = _parse_source_form_duid(d.pop("sourceFormDuid", UNSET))

        def _parse_created_by_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        created_by_duid = _parse_created_by_duid(d.pop("createdByDuid", UNSET))

        def _parse_updated_by_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_duid = _parse_updated_by_duid(d.pop("updatedByDuid", UNSET))

        def _parse_drafter_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        drafter_duid = _parse_drafter_duid(d.pop("drafterDuid", UNSET))

        in_trash = d.pop("inTrash", UNSET)

        dartboard_duid = d.pop("dartboardDuid", UNSET)

        order = d.pop("order", UNSET)

        expanded = d.pop("expanded", UNSET)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        description_markdown = d.pop("descriptionMarkdown", UNSET)

        status_duid = d.pop("statusDuid", UNSET)

        assigned_to_ai = d.pop("assignedToAi", UNSET)

        def _parse_recommendation_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        recommendation_duid = _parse_recommendation_duid(d.pop("recommendationDuid", UNSET))

        assignee_duids = cast(List[str], d.pop("assigneeDuids", UNSET))

        subscriber_duids = cast(List[str], d.pop("subscriberDuids", UNSET))

        tag_duids = cast(List[str], d.pop("tagDuids", UNSET))

        attachment_duids = cast(List[str], d.pop("attachmentDuids", UNSET))

        def _parse_priority(data: object) -> Union[None, Priority, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                priority_type_0 = Priority(data)

                return priority_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Priority, Unset], data)

        priority = _parse_priority(d.pop("priority", UNSET))

        def _parse_size(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        size = _parse_size(d.pop("size", UNSET))

        def _parse_start_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_at_type_0 = isoparse(data)

                return start_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        start_at = _parse_start_at(d.pop("startAt", UNSET))

        def _parse_due_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                due_at_type_0 = isoparse(data)

                return due_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        due_at = _parse_due_at(d.pop("dueAt", UNSET))

        def _parse_remind_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                remind_at_type_0 = isoparse(data)

                return remind_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        remind_at = _parse_remind_at(d.pop("remindAt", UNSET))

        def _parse_recurrence(data: object) -> Union[Any, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, None, Unset], data)

        recurrence = _parse_recurrence(d.pop("recurrence", UNSET))

        def _parse_recurs_next_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                recurs_next_at_type_0 = isoparse(data)

                return recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        recurs_next_at = _parse_recurs_next_at(d.pop("recursNextAt", UNSET))

        properties = d.pop("properties", UNSET)

        task_update = cls(
            duid=duid,
            source_type=source_type,
            source_template_view_duid=source_template_view_duid,
            source_template_task_duid=source_template_task_duid,
            source_form_duid=source_form_duid,
            created_by_duid=created_by_duid,
            updated_by_duid=updated_by_duid,
            drafter_duid=drafter_duid,
            in_trash=in_trash,
            dartboard_duid=dartboard_duid,
            order=order,
            expanded=expanded,
            title=title,
            description=description,
            description_markdown=description_markdown,
            status_duid=status_duid,
            assigned_to_ai=assigned_to_ai,
            recommendation_duid=recommendation_duid,
            assignee_duids=assignee_duids,
            subscriber_duids=subscriber_duids,
            tag_duids=tag_duids,
            attachment_duids=attachment_duids,
            priority=priority,
            size=size,
            start_at=start_at,
            due_at=due_at,
            remind_at=remind_at,
            recurrence=recurrence,
            recurs_next_at=recurs_next_at,
            properties=properties,
        )

        task_update.additional_properties = d
        return task_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
